<?php

namespace Drupal\datafield\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Url;
use Drupal\views\Views;

/**
 * Plugin implementation of the 'data_field' widget.
 *
 * @FieldWidget(
 *   id = "data_field_table_widget",
 *   label = @Translation("Data Field Table"),
 *   field_types = {"data_field"}
 * )
 */
class DataFieldTable extends DataField {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    $settings = [
      'field_reference' => '',
    ];
    return $settings + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $elements = parent::settingsForm($form, $form_state);
    $field_settings = $this->fieldDefinition->getSetting('field_settings');
    $storage_settings = $this->fieldDefinition->getFieldStorageDefinition()->getSetting('columns');
    $options = [];
    foreach ($storage_settings as $subfield => $columns) {
      if ($columns['type'] == 'entity_reference') {
        $options[$subfield] = $field_settings[$subfield]['label'];
      }
    }
    $field_reference = $this->getSetting('field_reference');
    if (!empty($options)) {
      $elements['field_reference'] = [
        '#type' => 'select',
        '#options' => $options,
        '#empty_option' => $this->t('- Select -'),
        '#title' => $this->t('Reference field'),
        '#description' => $this->t('Add a multiple search button on reference, only works with first column entity reference'),
        '#default_value' => $field_reference,
      ];
    }
    // Make the search entity reference the first column.
    if (!empty($field_reference)) {
      $elements["widget_settings"][$field_reference]["weight"]["#default_value"] = -1;
    }
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $storage = $this->getFormElement($items, $delta, $element, $form, $form_state);
    if (empty($storage)) {
      return [];
    }
    $cardinality = $this->fieldDefinition->getFieldStorageDefinition()->getCardinality();
    $widget = [
      '#type' => 'table',
    ];
    if ($cardinality != -1) {
      $widget += [
        '#header' => $this->buildTableHeader(),
        '#title' => $element["#title"],
        '#caption' => $element["#title"],
        '#attributes' => [
          'class' => ['caption-top'],
        ],
      ];
    }
    return $element + $widget + $storage;
  }

  /**
   * {@inheritDoc}
   */
  private function buildTableHeader() {
    $header = [];
    $subfields = $this->getFieldSetting('columns');
    $widgetSettings = $this->getSetting('widget_settings');
    if (!empty($widgetSettings)) {
      uasort($widgetSettings, [
        'Drupal\Component\Utility\SortArray',
        'sortByWeightElement',
      ]);
      $subfields = $widgetSettings;
    }
    $fieldSettings = $this->getFieldSetting('field_settings');
    foreach (array_keys($subfields) as $subfield) {
      $item = $this->getFieldSetting('columns')[$subfield];
      if (!empty($widgetSettings[$subfield]) && empty($widgetSettings[$subfield]['field_display'])) {
        continue;
      }
      if (!empty($widgetSettings[$subfield]) && $widgetSettings[$subfield]['label_display'] == 'hidden') {
        $header[] = '';
        continue;
      }
      $header[] = $fieldSettings[$subfield]['label'] ?? $item['name'];
    }
    $settings = $this->getSettings();
    if (!empty($settings['operation'])) {
      $header[] = '';
    }
    return $header;
  }

  /**
   * {@inheritDoc}
   */
  public function formMultipleElements(FieldItemListInterface $items, array &$form, FormStateInterface $form_state) {
    $elements = parent::formMultipleElements($items, $form, $form_state);
    if (!empty($field_reference = $this->getSetting('field_reference'))) {
      $fieldDefinition = $this->fieldDefinition->getSetting('field_settings');
      $elements['#field_reference'] = $field_reference;
      $elements['#field_reference_title'] = str_replace('"', '', $fieldDefinition[$field_reference]['label']);
      $fieldSettings = $this->getFieldSetting('field_settings')[$field_reference];
      $elements['#entity_reference_type'] = $fieldSettings['entity_reference_type'];
      $elements['#target_bundles'] = $fieldSettings['target_bundles'];
      $elements['#field_reference_url'] = Url::fromRoute('datafield.field_reference', [
        'entity_type' => $fieldSettings['entity_reference_type'],
        'bundle' => $fieldSettings['target_bundles'],
        'field_reference' => $field_reference,
      ])->toString();
      $elements['#attached']['drupalSettings']['field_reference'][$field_reference] = $this->fieldReferenceSettings($fieldSettings['target_bundles']);
    }
    $subFormElements = Element::children($elements);
    foreach ($subFormElements as $delta) {
      if (is_numeric($delta)) {
        $childElements = Element::children($elements[$delta]);
        $element = [];
        foreach ($childElements as $child) {
          if (is_numeric($child)) {
            $element = $elements[$delta][$child];
          }
          else {
            $element[$child] = $elements[$delta][$child];
          }
        }
        unset($elements[$delta]);
        $elements[$delta] = $element;
      }
    }
    $elements += [
      '#widgetDataFieldTable' => TRUE,
    ];
    return $elements;
  }

  /**
   * Set drupal settings for search entity.
   *
   * @param string $target_bundle
   *   The bundle select or views.
   *
   * @return array
   *   The settings columns.
   */
  protected function fieldReferenceSettings(string $target_bundle): array {
    $settings = [
      'columns' => [
        ['field' => 'state', 'checkbox' => "true"],
        ['field' => 'id', 'title' => $this->t("id")],
      ],
    ];
    $checkView = explode(':', $target_bundle);
    if (!empty($checkView[1])) {
      $view = Views::getView($checkView[0]);
      $view->setDisplay($checkView[1]);
      $displayHandler = $view->displayHandlers->get($checkView[1]);
      $settings['style'] = $displayHandler->getPlugin('style')->options;
      $settings['options'] = $displayHandler->getPlugin('row')->options;
      $fields = $displayHandler->getOption('fields');
      foreach ($fields as $fieldId => $field) {
        $settings['columns'][] = [
          'field' => $fieldId,
          'title' => !empty($field['label']) ? $field['label'] : $field["entity_field"],
          'sortable' => "true",
        ];
      }
    }
    else {
      $settings['columns'][] = [
        'name' => 'name',
        'title' => $this->t('Name'),
      ];
    }
    return $settings;
  }

}
